@@ -72,6 +72,7 @@ bluetooth_pairing_dialog_set_mode (BluetoothPairingDialog *self,
 	g_clear_pointer (&priv->pin, g_free);
 	priv->pin = g_strdup (pin);
 	gtk_editable_set_text (GTK_EDITABLE (priv->entry_pin), pin ? pin : "");
+	gtk_entry_set_visibility (GTK_ENTRY (priv->entry_pin), TRUE);
 	gtk_label_set_text (GTK_LABEL (priv->label_pin), pin);
 
 	switch (mode) {
@@ -124,6 +125,8 @@ bluetooth_pairing_dialog_set_mode (BluetoothPairingDialog *self,
 		break;
 	case BLUETOOTH_PAIRING_MODE_PIN_DISPLAY_KEYBOARD:
 		help = g_strdup_printf (_("Please enter the following PIN on “%s”. Then press “Return” on the keyboard."), device_name);
+		/* populate the invisible character */
+		gtk_entry_set_visibility (GTK_ENTRY (priv->entry_pin), FALSE);
 		break;
 	case BLUETOOTH_PAIRING_MODE_PIN_DISPLAY_ICADE:
 		help = g_strdup (_("Please move the joystick of your iCade in the following directions. Then press any of the white buttons."));
@@ -184,7 +187,7 @@ bluetooth_pairing_dialog_set_pin_entered (BluetoothPairingDialog *self,
 					  guint                   entered)
 {
 	BluetoothPairingDialogPrivate *priv = BLUETOOTH_PAIRING_DIALOG_GET_PRIVATE (self);
-	char *done;
+	g_autofree char *done = NULL;
 
 	g_assert (priv->mode == BLUETOOTH_PAIRING_MODE_PIN_DISPLAY_KEYBOARD);
 	g_assert (priv->pin);
@@ -195,6 +198,8 @@ bluetooth_pairing_dialog_set_pin_entered (BluetoothPairingDialog *self,
 		guint i;
 
 		invisible = gtk_entry_get_invisible_char (GTK_ENTRY (priv->entry_pin));
+		if (invisible == 0)
+			invisible = '*';
 
 		str = g_string_new (NULL);
 		for (i = 0; i < entered; i++)
@@ -208,7 +213,6 @@ bluetooth_pairing_dialog_set_pin_entered (BluetoothPairingDialog *self,
 	}
 
 	gtk_label_set_text (GTK_LABEL (priv->label_pin), done);
-	g_free (done);
 }
 
 static void
@@ -250,7 +254,7 @@ text_changed_cb (GObject    *gobject,
 static void
 bluetooth_pairing_dialog_init (BluetoothPairingDialog *self)
 {
-	GtkCssProvider *provider;
+	g_autoptr(GtkCssProvider) provider = NULL;
 
 	gtk_widget_init_template (GTK_WIDGET (self));
 
@@ -262,8 +266,6 @@ bluetooth_pairing_dialog_init (BluetoothPairingDialog *self)
 	gtk_style_context_add_provider_for_display (gdk_display_get_default (),
 						    GTK_STYLE_PROVIDER (provider),
 						    GTK_STYLE_PROVIDER_PRIORITY_USER);
-	g_object_unref (provider);
-
 }
 
 static void
