@@ -28,13 +28,14 @@
 
 #include <sys/time.h>
 
+#include <adwaita.h>
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
 #include <gio/gio.h>
 #include <gtk/gtk.h>
 
 #include <bluetooth-client.h>
-#include <bluetooth-chooser.h>
+#include <bluetooth-device.h>
 
 #define OBEX_SERVICE	"org.bluez.obex"
 #define OBEX_PATH	"/org/bluez/obex"
@@ -428,8 +429,7 @@ static void response_callback(GtkWidget *dialog,
 		current_transfer = NULL;
 	}
 
-	gtk_widget_destroy(dialog);
-	gtk_main_quit();
+	gtk_window_destroy(GTK_WINDOW (dialog));
 }
 
 static void create_window(void)
@@ -448,116 +448,106 @@ static void create_window(void)
 			       _("_Retry"), RESPONSE_RETRY,
 			       NULL);
 	gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog), RESPONSE_RETRY, FALSE);
-	gtk_window_set_type_hint(GTK_WINDOW(dialog),
-				 GDK_WINDOW_TYPE_HINT_NORMAL);
-	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
 	gtk_window_set_default_size(GTK_WINDOW(dialog), 400, -1);
-	gtk_container_set_border_width(GTK_CONTAINER(dialog), 6);
 
 	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
+	gtk_widget_set_margin_end(vbox, 12);
+	gtk_widget_set_margin_start(vbox, 12);
+	gtk_widget_set_margin_top(vbox, 12);
+	gtk_widget_set_margin_bottom(vbox, 12);
 	gtk_box_set_spacing(GTK_BOX(vbox), 6);
-	gtk_container_set_border_width(GTK_CONTAINER(vbox), 6);
-	gtk_container_add (GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
-	                   vbox);
+	gtk_window_set_child (GTK_WINDOW(dialog), vbox);
 
 	table = gtk_grid_new();
 	gtk_grid_set_column_spacing(GTK_GRID(table), 4);
 	gtk_grid_set_row_spacing(GTK_GRID(table), 4);
-	gtk_box_pack_start(GTK_BOX(vbox), table, TRUE, TRUE, 9);
+	gtk_box_append(GTK_BOX(vbox), table);
 
 	label = gtk_label_new(NULL);
-	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
+	gtk_label_set_xalign(GTK_LABEL(label), 1.0);
+	gtk_label_set_yalign(GTK_LABEL(label), 0.5);
 	text = g_markup_printf_escaped("<b>%s</b>", _("From:"));
 	gtk_label_set_markup(GTK_LABEL(label), text);
 	g_free(text);
 	gtk_grid_attach(GTK_GRID(table), label, 0, 0, 1, 1);
 
 	label_from = gtk_label_new(NULL);
-	gtk_misc_set_alignment(GTK_MISC(label_from), 0, 0.5);
+	gtk_label_set_xalign(GTK_LABEL(label_from), 0.0);
+	gtk_label_set_yalign(GTK_LABEL(label_from), 0.5);
 	gtk_label_set_ellipsize(GTK_LABEL(label_from), PANGO_ELLIPSIZE_MIDDLE);
 	gtk_grid_attach(GTK_GRID(table), label_from, 1, 0, 1, 1);
 
 	update_from_label ();
 
 	label = gtk_label_new(NULL);
-	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
+	gtk_label_set_xalign(GTK_LABEL(label), 1.0);
+	gtk_label_set_yalign(GTK_LABEL(label), 0.5);
 	text = g_markup_printf_escaped("<b>%s</b>", _("To:"));
 	gtk_label_set_markup(GTK_LABEL(label), text);
 	g_free(text);
 	gtk_grid_attach(GTK_GRID(table), label, 0, 1, 1, 1);
 
 	label = gtk_label_new(NULL);
-	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
+	gtk_label_set_xalign(GTK_LABEL(label), 0.0);
+	gtk_label_set_yalign(GTK_LABEL(label), 0.5);
 	gtk_label_set_ellipsize(GTK_LABEL(label), PANGO_ELLIPSIZE_END);
 	gtk_label_set_text(GTK_LABEL(label), option_device_name);
 	gtk_grid_attach(GTK_GRID(table), label, 1, 1, 1, 1);
 
 	progress = gtk_progress_bar_new();
+	gtk_widget_set_vexpand (progress, TRUE);
 	gtk_progress_bar_set_show_text (GTK_PROGRESS_BAR (progress), TRUE);
 	gtk_progress_bar_set_ellipsize(GTK_PROGRESS_BAR(progress),
 							PANGO_ELLIPSIZE_END);
 	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(progress),
 							_("Connecting…"));
-	gtk_box_pack_start(GTK_BOX(vbox), progress, TRUE, TRUE, 0);
+	gtk_box_append(GTK_BOX(vbox), progress);
 
 	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 4);
 
-	image_status = gtk_image_new_from_icon_name ("dialog-warning", GTK_ICON_SIZE_MENU);
-	gtk_widget_set_no_show_all (image_status, TRUE);
-	gtk_box_pack_start(GTK_BOX (hbox), image_status, FALSE, FALSE, 4);
+	image_status = gtk_image_new_from_icon_name ("dialog-warning");
+	gtk_widget_hide(image_status);
+	gtk_box_append(GTK_BOX(hbox), image_status);
 
 	label_status = gtk_label_new(NULL);
-	gtk_misc_set_alignment(GTK_MISC(label_status), 0, 0.5);
-	gtk_label_set_line_wrap(GTK_LABEL(label_status), TRUE);
-	gtk_box_pack_start(GTK_BOX (hbox), label_status, TRUE, TRUE, 4);
+	gtk_label_set_xalign(GTK_LABEL(label_status), 0.0);
+	gtk_label_set_yalign(GTK_LABEL(label_status), 0.5);
+	gtk_label_set_wrap(GTK_LABEL(label_status), TRUE);
+	gtk_box_append(GTK_BOX(hbox), label_status);
 
-	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 2);
+	gtk_box_append(GTK_BOX(vbox), hbox);
 
 	g_signal_connect(G_OBJECT(dialog), "response",
 				G_CALLBACK(response_callback), NULL);
 
-	gtk_widget_show_all(dialog);
+	gtk_window_present(GTK_WINDOW(dialog));
 }
 
-static gchar *get_device_name(const gchar *address)
+static char *
+get_device_name (const char *address)
 {
-	BluetoothClient *client;
-	GtkTreeModel *model;
-	GtkTreeIter iter;
-	gboolean cont;
-	char *found_name;
+	g_autoptr(BluetoothClient) client = NULL;
+	g_autoptr(GListStore) model = NULL;
+	guint n_devices, i;
 
-	found_name = NULL;
-	client = bluetooth_client_new ();
-	model = bluetooth_client_get_model (client);
-	if (model == NULL) {
-		g_object_unref (client);
-		return NULL;
+	model = bluetooth_client_get_devices (client);
+	n_devices = g_list_model_get_n_items (G_LIST_MODEL (model));
+	for (i = 0; i < n_devices; i++) {
+		g_autoptr(BluetoothDevice) device = NULL;
+		g_autofree char *name = NULL;
+		g_autofree char *_address = NULL;
+
+		g_object_get (device,
+			      "name", &name,
+			      "address", &_address,
+			      NULL);
+		if (g_strcmp0 (address, _address) != 0)
+			continue;
+
+		return g_steal_pointer (&name);
 	}
 
-	cont = gtk_tree_model_get_iter_first(model, &iter);
-	while (cont != FALSE) {
-		char *bdaddr, *name;
-
-		gtk_tree_model_get(model, &iter,
-				   BLUETOOTH_COLUMN_ADDRESS, &bdaddr,
-				   BLUETOOTH_COLUMN_ALIAS, &name,
-				   -1);
-		if (g_strcmp0 (bdaddr, address) == 0) {
-			g_free (bdaddr);
-			found_name = name;
-			break;
-		}
-		g_free (bdaddr);
-		g_free (name);
-
-		cont = gtk_tree_model_iter_next(model, &iter);
-	}
-
-	g_object_unref (model);
-	g_object_unref (client);
-
-	return found_name;
+	return NULL;
 }
 
 static void
@@ -684,138 +674,59 @@ on_transfer_error (void)
 	current_transfer = NULL;
 }
 
-static void
-select_device_changed(BluetoothChooser *sel,
-		      char *address,
-		      gpointer user_data)
+static gint select_dialog_response;
+static GMainLoop *select_dialog_mainloop = NULL;
+
+static void select_dialog_response_callback(GtkWidget *dialog,
+					gint response, gpointer user_data)
 {
-	GtkDialog *dialog = user_data;
-	char *icon;
-
-	if (address == NULL)
-		goto bail;
-
-	icon = bluetooth_chooser_get_selected_device_icon (sel);
-	if (icon == NULL)
-		goto bail;
-
-	/* Apple's device don't have OBEX */
-	if (g_str_equal (icon, "phone-apple-iphone"))
-		goto bail;
-
-	gtk_dialog_set_response_sensitive (dialog,
-					   GTK_RESPONSE_ACCEPT, TRUE);
-	return;
-
-bail:
-	gtk_dialog_set_response_sensitive (dialog,
-					   GTK_RESPONSE_ACCEPT, FALSE);
+	select_dialog_response = response;
+	g_main_loop_quit(select_dialog_mainloop);
 }
-
-static void
-select_device_activated(BluetoothChooser *sel,
-			char *address,
-			gpointer user_data)
-{
-	GtkDialog *dialog = user_data;
-
-	gtk_dialog_response(dialog, GTK_RESPONSE_ACCEPT);
-}
-
-static char *
-show_browse_dialog (char **device_name)
-{
-	GtkWidget *dialog, *selector, *send_button, *content_area;
-	char *bdaddr;
-	int response_id;
-	GtkStyleContext *context;
-
-	dialog = g_object_new (GTK_TYPE_DIALOG,
-			       "title", _("Select device to send to"),
-			       "use-header-bar", 1,
-			       NULL);
-	gtk_dialog_add_buttons(GTK_DIALOG (dialog),
-			       _("_Cancel"), GTK_RESPONSE_CANCEL,
-			       _("_Send"), GTK_RESPONSE_ACCEPT,
-			       NULL);
-	gtk_window_set_type_hint (GTK_WINDOW (dialog), GDK_WINDOW_TYPE_HINT_NORMAL);
-	send_button = gtk_dialog_get_widget_for_response(GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
-	context = gtk_widget_get_style_context(send_button);
-	gtk_style_context_add_class (context, "suggested-action");
-
-	gtk_dialog_set_response_sensitive(GTK_DIALOG(dialog),
-					  GTK_RESPONSE_ACCEPT, FALSE);
-	gtk_window_set_default_size(GTK_WINDOW(dialog), 480, 400);
-
-	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
-	content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
-	gtk_box_set_spacing (GTK_BOX (content_area), 2);
-
-	selector = bluetooth_chooser_new();
-	gtk_container_set_border_width(GTK_CONTAINER(selector), 5);
-	gtk_widget_show(selector);
-	g_object_set(selector,
-		     "show-searching", TRUE,
-		     "show-device-category", TRUE,
-		     "show-device-type", TRUE,
-		     NULL);
-	g_signal_connect(selector, "selected-device-changed",
-			 G_CALLBACK(select_device_changed), dialog);
-	g_signal_connect(selector, "selected-device-activated",
-			 G_CALLBACK(select_device_activated), dialog);
-	gtk_box_pack_start (GTK_BOX (content_area), selector, TRUE, TRUE, 0);
-	bluetooth_chooser_start_discovery (BLUETOOTH_CHOOSER (selector));
-
-	bdaddr = NULL;
-	response_id = gtk_dialog_run (GTK_DIALOG (dialog));
-	if (response_id == GTK_RESPONSE_ACCEPT) {
-		bdaddr = bluetooth_chooser_get_selected_device (BLUETOOTH_CHOOSER (selector));
-		*device_name = bluetooth_chooser_get_selected_device_name (BLUETOOTH_CHOOSER (selector));
-	}
-
-	gtk_widget_destroy (dialog);
-
-	return bdaddr;
-}
-
 static char **
 show_select_dialog(void)
 {
 	GtkWidget *dialog, *button;
 	gchar **files = NULL;
-	GtkStyleContext *context;
 
 	dialog = g_object_new (GTK_TYPE_FILE_CHOOSER_DIALOG,
 			       "title", _("Choose files to send"),
 			       "action", GTK_FILE_CHOOSER_ACTION_OPEN,
 			       "use-header-bar", 1,
+			       "hide-on-close", TRUE,
 			       NULL);
 	gtk_dialog_add_buttons(GTK_DIALOG (dialog),
 			       _("_Cancel"), GTK_RESPONSE_CANCEL,
 			       _("Select"), GTK_RESPONSE_ACCEPT, NULL);
-	gtk_window_set_type_hint (GTK_WINDOW (dialog), GDK_WINDOW_TYPE_HINT_NORMAL);
 	gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(dialog), TRUE);
 
 	button = gtk_dialog_get_widget_for_response(GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
-	context = gtk_widget_get_style_context(button);
-	gtk_style_context_add_class (context, "suggested-action");
+	gtk_widget_add_css_class(button, "suggested-action");
 
-	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
-		GSList *list, *filenames;
-		int i;
+	g_signal_connect(dialog, "response", G_CALLBACK(select_dialog_response_callback), NULL);
 
-		filenames = gtk_file_chooser_get_filenames(GTK_FILE_CHOOSER(dialog));
+	gtk_widget_show(dialog);
 
-		files = g_new(gchar *, g_slist_length(filenames) + 1);
+	select_dialog_mainloop = g_main_loop_new(NULL, FALSE);
+	g_main_loop_run(select_dialog_mainloop);
 
-		for (list = filenames, i = 0; list; list = list->next, i++)
-			files[i] = list->data;
+	if (select_dialog_response == GTK_RESPONSE_ACCEPT) {
+		g_autoptr(GListModel) selected_files = NULL;
+		guint i, n_items;
+
+		selected_files = gtk_file_chooser_get_files(GTK_FILE_CHOOSER(dialog));
+		n_items = g_list_model_get_n_items(selected_files);
+
+		files = g_new(gchar *, n_items + 1);
+
+		for (i = 0; i < n_items; i++) {
+			g_autoptr(GFile) file = g_list_model_get_item(selected_files, i);
+			files[i] = g_file_get_path(file);
+		}
 		files[i] = NULL;
-
-		g_slist_free(filenames);
 	}
 
-	gtk_widget_destroy(dialog);
+	gtk_window_destroy(GTK_WINDOW(dialog));
 
 	return files;
 }
@@ -834,6 +745,7 @@ static GOptionEntry options[] = {
 
 int main(int argc, char *argv[])
 {
+	g_autoptr(GOptionContext) option_context = NULL;
 	GError *error = NULL;
 	int i;
 
@@ -843,8 +755,12 @@ int main(int argc, char *argv[])
 
 	error = NULL;
 
-	if (gtk_init_with_args(&argc, &argv, _("[FILE…]"),
-				options, GETTEXT_PACKAGE, &error) == FALSE) {
+	gtk_init();
+	adw_init();
+
+	option_context = g_option_context_new(NULL);
+	g_option_context_add_main_entries(option_context, options, GETTEXT_PACKAGE);
+	if (g_option_context_parse(option_context, &argc, &argv, &error) == FALSE) {
 		if (error != NULL) {
 			g_printerr("%s\n", error->message);
 			g_error_free(error);
@@ -872,13 +788,8 @@ int main(int argc, char *argv[])
 			return 1;
 	}
 
-	if (option_device == NULL) {
-		option_device = show_browse_dialog(&option_device_name);
-		if (option_device == NULL) {
-			g_strfreev(option_files);
-			return 1;
-		}
-	}
+	if (option_device == NULL)
+		return 1;
 
 	file_count = g_strv_length(option_files);
 
@@ -941,7 +852,8 @@ int main(int argc, char *argv[])
 	if (!g_cancellable_is_cancelled (cancellable))
 		send_files ();
 
-	gtk_main();
+	while (g_list_model_get_n_items (gtk_window_get_toplevels()) > 0)
+		g_main_context_iteration (NULL, TRUE);
 
 	g_cancellable_cancel (cancellable);
 
